/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NUnit.Framework;
using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.NUnit2.Types;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Tests the functionality and error cases of <c>ServerAppTestTask</c> class. Since attributes are important to the
    /// functionality of the class, they are also tested. Method <c>ExecuteTask()</c> is protected. Therefore, it cannot
    /// be tested directly. Instead, it is tested by <c>Execute()</c> method, since this method is largely a wrapper of
    /// the <c>ExecuteTask()</c> method.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ServerAppTestTaskTestCase
    {
        /// <summary>
        /// Represents the <c>ServerAppTestTask</c> instance used in tests.
        /// </summary>
        private ServerAppTestTask task;

        /// <summary>
        /// Represents the NAnt project used in tests.
        /// </summary>
        private Project project;

        /// <summary>
        /// Asserts the calls to the mock server. The calls to <c>StartUp()</c>, <c>ShutDown()</c>, <c>InitData()</c>,
        /// and <c>CleanUpData()</c> should be as expected.
        /// </summary>
        /// <param name="server">The mock server instance to be verified.</param>
        /// <param name="started">A flag indicating whether <c>StartUp()</c> should be called.</param>
        /// <param name="stopped">A flag indicating whether <c>ShutDown()</c> should be called.</param>
        /// <param name="initialized">A flag indicating whether <c>InitData()</c> should be called.</param>
        /// <param name="cleaned">A flag indicating whether <c>CleanUpData()</c> should be called.</param>
        private void AssertServerCall(MockServer server, bool started, bool stopped, bool initialized, bool cleaned)
        {
            Assert.AreEqual(started, server.Started, "The starting status of the server should be as expected.");
            Assert.AreEqual(stopped, server.Stopped, "The stopping status of the server should be as expected.");
            Assert.AreEqual(initialized, server.Initialized,
                "The initializing status of the server should be as expected.");
            Assert.AreEqual(cleaned, server.Cleaned, "The cleaning status of the server should be as expected.");
        }

        /// <summary>
        /// Sets up the test environment. The test instance is created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            task = new ServerAppTestTask();
            project = new Project("../../test_files/default.build", Level.Error);
            task.Project = project;
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            project = null;
            task = null;
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>ServerAppTestTask</c> class. A <c>TaskName</c> attribute with
        /// name "serverapptest" should exist.
        /// </summary>
        [Test]
        public void TestServerAppTestTaskClassAttributeAccuracy()
        {
            TaskNameAttribute attribute =
                (TaskNameAttribute)typeof(ServerAppTestTask).GetCustomAttributes(typeof(TaskNameAttribute), false)[0];

            // Verify
            Assert.AreEqual("serverapptest", attribute.Name, "The name of the attribute must be serverapptest.");
        }

        /// <summary>
        /// Tests accuracy of <c>ServerAppTestTask()</c>. Since the constructor is trivial, there should be no
        /// exception.
        /// </summary>
        [Test]
        public void TestServerAppTestTaskAccuracy()
        {
            Assert.IsNotNull(new ServerAppTestTask(), "A new instance of ServerAppTestTask should be created.");
        }

        /// <summary>
        /// Tests accuracy of <c>Servers</c> for the default value. By default, it is <c>null</c>. This is verified by
        /// the property getter.
        /// </summary>
        [Test]
        public void TestServersDefaultAccuracy()
        {
            Assert.IsNull(task.Servers, "The default value of server array should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Servers</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestServersSetAccuracy()
        {
            ServerElement[] servers = new ServerElement[10];

            // Set the value
            task.Servers = servers;

            // Verify
            Assert.AreEqual(servers, task.Servers, "The server array should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>Servers</c> property. A <c>BuildElementArray</c> attribute with
        /// name "server" should exist. It should be a required attribute.
        /// </summary>
        [Test]
        public void TestServersAttributeAccuracy()
        {
            BuildElementArrayAttribute attribute = (BuildElementArrayAttribute)typeof(ServerAppTestTask).
                GetProperty("Servers").GetCustomAttributes(typeof(BuildElementArrayAttribute), false)[0];

            // Verify
            Assert.AreEqual("server", attribute.Name, "The name of the attribute must be server.");
            Assert.IsTrue(attribute.Required, "The attribute must be required.");
        }

        /// <summary>
        /// Tests <c>ExecuteTask()</c> when starting up a server fails. All other servers should be started and then
        /// stopped, except servers which does not need to be started. This is verified by the mock servers. Expect
        /// ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTaskStartUpError()
        {
            // Create the array of server elements
            ServerElement[] elements =
                new ServerElement[3] { new ServerElement(), new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = typeof(StartErrorServer).FullName;
            elements[2].ClassName = typeof(MockServer).FullName;
            elements[2].StartServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Execute the task
            try
            {
                task.Execute();
            }
            finally
            {
                // Verify the calls to servers
                AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, false, false);
                AssertServerCall((MockServer)elements[1].GetServerInstance(), true, false, false, false);
                AssertServerCall((MockServer)elements[2].GetServerInstance(), false, false, false, false);
            }
        }

        /// <summary>
        /// Tests <c>ExecuteTask()</c> when a server cannot be created from the server element. All other servers should
        /// be started and then stopped, except servers which does not need to be started. This is verified by the mock
        /// servers. Expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTaskServerElementError()
        {
            // Create the array of server elements
            ServerElement[] elements =
                new ServerElement[3] { new ServerElement(), new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = "unknown";
            elements[2].ClassName = typeof(MockServer).FullName;
            elements[2].StartServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Execute the task
            try
            {
                task.Execute();
            }
            finally
            {
                // Verify the calls to servers
                AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, false, false);
                AssertServerCall((MockServer)elements[2].GetServerInstance(), false, false, false, false);
            }
        }

        /// <summary>
        /// Tests <c>ExecuteTask()</c> when initializing a server fails. All other servers should be started,
        /// initialized, cleaned and then stopped, except servers which does not need to be started/initialized. This is
        /// verified by the mock servers. Expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTaskInitDataError()
        {
            // Create the array of server elements
            ServerElement[] elements = new ServerElement[] { new ServerElement(), new ServerElement(),
                new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = typeof(MockServer).FullName;
            elements[1].StartServer = false;
            elements[2].ClassName = typeof(InitErrorServer).FullName;
            elements[3].ClassName = typeof(MockServer).FullName;
            elements[3].InitServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Execute the task
            try
            {
                task.Execute();
            }
            finally
            {
                // Verify the calls to servers
                AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, true, true);
                AssertServerCall((MockServer)elements[1].GetServerInstance(), false, false, true, true);
                AssertServerCall((MockServer)elements[2].GetServerInstance(), true, true, true, false);
                AssertServerCall((MockServer)elements[3].GetServerInstance(), true, true, false, false);
            }
        }

        /// <summary>
        /// Tests <c>ExecuteTask()</c> when the tests cause an exception. All servers should be started, initialized,
        /// cleaned and then stopped, except servers which does not need to be started/initialized. This is
        /// verified by the mock servers. Expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTaskTestError()
        {
            // Create the array of server elements
            ServerElement[] elements =
                new ServerElement[] { new ServerElement(), new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = typeof(MockServer).FullName;
            elements[1].StartServer = false;
            elements[2].ClassName = typeof(MockServer).FullName;
            elements[2].InitServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Add a test
            NUnit2Test test = new NUnit2Test();

            test.AssemblyName = "unknown";
            test.Project = project;
            task.Tests.Add(test);

            // Execute the task
            try
            {
                task.Execute();
            }
            finally
            {
                // Verify the calls to servers
                AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, true, true);
                AssertServerCall((MockServer)elements[1].GetServerInstance(), false, false, true, true);
                AssertServerCall((MockServer)elements[2].GetServerInstance(), true, true, false, false);
            }
        }

        /// <summary>
        /// Tests <c>ExecuteTask()</c> when cleaning a server fails. It should be stopped. All other servers should be
        /// started, initialized, cleaned and then stopped, except servers which does not need to be
        /// started/initialized. This is verified by the mock servers. Expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTaskCleanError()
        {
            // Create the array of server elements
            ServerElement[] elements = new ServerElement[] { new ServerElement(), new ServerElement(),
                new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = typeof(MockServer).FullName;
            elements[1].StartServer = false;
            elements[2].ClassName = typeof(CleanErrorServer).FullName;
            elements[3].ClassName = typeof(MockServer).FullName;
            elements[3].InitServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Execute the task
            try
            {
                task.Execute();
            }
            finally
            {
                // Verify the calls to servers
                AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, true, true);
                AssertServerCall((MockServer)elements[1].GetServerInstance(), false, false, true, true);
                AssertServerCall((MockServer)elements[2].GetServerInstance(), true, true, true, true);
                AssertServerCall((MockServer)elements[3].GetServerInstance(), true, true, false, false);
            }
        }

        /// <summary>
        /// Tests <c>ExecuteTask()</c> when stopping a server fails. All other servers should be started, initialized,
        /// cleaned and then stopped, except servers which does not need to be started/initialized. This is verified by
        /// the mock servers. Expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTaskStopError()
        {
            // Create the array of server elements
            ServerElement[] elements = new ServerElement[] { new ServerElement(), new ServerElement(),
                new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = typeof(MockServer).FullName;
            elements[1].StartServer = false;
            elements[2].ClassName = typeof(StopErrorServer).FullName;
            elements[3].ClassName = typeof(MockServer).FullName;
            elements[3].InitServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Execute the task
            try
            {
                task.Execute();
            }
            finally
            {
                // Verify the calls to servers
                AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, true, true);
                AssertServerCall((MockServer)elements[1].GetServerInstance(), false, false, true, true);
                AssertServerCall((MockServer)elements[2].GetServerInstance(), true, true, true, true);
                AssertServerCall((MockServer)elements[3].GetServerInstance(), true, true, false, false);
            }
        }

        /// <summary>
        /// Tests accuracy of <c>ExecuteTask()</c>. The server should be started/stop and/or initialized/cleaned
        /// according to the server/init server flags. These are verified by the mock server instances.
        /// </summary>
        public void TestExecuteTaskAccuracy()
        {
            // Create the array of server elements
            ServerElement[] elements =
                new ServerElement[] { new ServerElement(), new ServerElement(), new ServerElement() };

            elements[0].ClassName = typeof(MockServer).FullName;
            elements[1].ClassName = typeof(MockServer).FullName;
            elements[1].StartServer = false;
            elements[2].ClassName = typeof(MockServer).FullName;
            elements[2].InitServer = false;

            // Set the property of the task
            task.Servers = elements;

            // Execute the task
            task.Execute();

            // Verify the calls to servers
            AssertServerCall((MockServer)elements[0].GetServerInstance(), true, true, true, true);
            AssertServerCall((MockServer)elements[1].GetServerInstance(), false, false, true, true);
            AssertServerCall((MockServer)elements[2].GetServerInstance(), true, true, false, false);
        }
    }
}

