/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NUnit.Framework;
using NAnt.Core;
using NAnt.Core.Types;
using NAnt.Core.Attributes;
using TopCoder.Test.Framework.Servers;
using TopCoder.Test.Framework.Servers.Web;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Tests the functionality and error cases of <c>ServerElement</c> class. Since attributes are important to the
    /// functionality of the class, they are also tested.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ServerElementTestCase
    {
        /// <summary>
        /// Represents the <c>ServerElement</c> instance used in tests.
        /// </summary>
        private ServerElement element;

        /// <summary>
        /// Sets up the test environment. The test instance is created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            element = new ServerElement();
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            element = null;
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>ServerElement</c> class. An <c>ElementName</c> attribute with
        /// name "server" should exist.
        /// </summary>
        [Test]
        public void TestServerElementClassAttributeAccuracy()
        {
            ElementNameAttribute attribute =
                (ElementNameAttribute)typeof(ServerElement).GetCustomAttributes(typeof(ElementNameAttribute), false)[0];

            // Verify
            Assert.AreEqual("server", attribute.Name, "The name of the attribute must be server.");
        }

        /// <summary>
        /// Tests accuracy of <c>ClassName</c> for the default value. By default, it is <c>null</c>. This is verified
        /// by the property getter.
        /// </summary>
        [Test]
        public void TestClassNameDefaultAccuracy()
        {
            Assert.IsNull(element.ClassName, "The default value of class name should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>ClassName</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestClassNameSetAccuracy()
        {
            element.ClassName = "any class name";

            // Verify
            Assert.AreEqual("any class name", element.ClassName, "The class name should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>ClassName</c> property. A <c>TaskAttribute</c> attribute with
        /// name "class" should exist. It should be a required attribute.
        /// </summary>
        [Test]
        public void TestClassNameAttributeAccuracy()
        {
            TaskAttributeAttribute attribute = (TaskAttributeAttribute)typeof(ServerElement).GetProperty("ClassName").
                GetCustomAttributes(typeof(TaskAttributeAttribute), false)[0];

            // Verify
            Assert.AreEqual("class", attribute.Name, "The name of the attribute must be class.");
            Assert.IsTrue(attribute.Required, "The attribute must be required.");
        }

        /// <summary>
        /// Tests accuracy of <c>StartServer</c> for the default value. By default, it is <c>true</c>. This is verified
        /// by the property getter.
        /// </summary>
        [Test]
        public void TestStartServerDefaultAccuracy()
        {
            Assert.IsTrue(element.StartServer, "The default value of start server flag should be true.");
        }

        /// <summary>
        /// Tests accuracy of <c>StartServer</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestStartServerSetAccuracy()
        {
            element.StartServer = false;

            // Verify
            Assert.IsFalse(element.StartServer, "The start server flag should be set as false.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>StartServer</c> property. A <c>TaskAttribute</c> attribute with
        /// name "startserver" should exist. It should be an optional attribute.
        /// </summary>
        [Test]
        public void TestStartServerAttributeAccuracy()
        {
            TaskAttributeAttribute attribute = (TaskAttributeAttribute)typeof(ServerElement).GetProperty("StartServer").
                GetCustomAttributes(typeof(TaskAttributeAttribute), false)[0];

            // Verify
            Assert.AreEqual("startserver", attribute.Name, "The name of the attribute must be startserver.");
            Assert.IsFalse(attribute.Required, "The attribute must be optional.");
        }

        /// <summary>
        /// Tests accuracy of <c>InitServer</c> for the default value. By default, it is <c>true</c>. This is verified
        /// by the property getter.
        /// </summary>
        [Test]
        public void TestInitServerDefaultAccuracy()
        {
            Assert.IsTrue(element.InitServer, "The default value of init data flag should be true.");
        }

        /// <summary>
        /// Tests accuracy of <c>InitServer</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestInitServerSetAccuracy()
        {
            element.InitServer = false;

            // Verify
            Assert.IsFalse(element.InitServer, "The init data flag should be set as false.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>InitServer</c> property. A <c>TaskAttribute</c> attribute with
        /// name "initserver" should exist. It should be an optional attribute.
        /// </summary>
        [Test]
        public void TestInitServerAttributeAccuracy()
        {
            TaskAttributeAttribute attribute = (TaskAttributeAttribute)typeof(ServerElement).GetProperty("InitServer").
                GetCustomAttributes(typeof(TaskAttributeAttribute), false)[0];

            // Verify
            Assert.AreEqual("initserver", attribute.Name, "The name of the attribute must be initserver.");
            Assert.IsFalse(attribute.Required, "The attribute must be optional.");
        }

        /// <summary>
        /// Tests accuracy of <c>Options</c> for the default value. By default, it is not <c>null</c>. This is verified
        /// by the property getter.
        /// </summary>
        [Test]
        public void TestOptionsDefaultAccuracy()
        {
            Assert.IsNotNull(element.Options, "The default value of options should not be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Options</c> property getter. It should return the same instance each time.
        /// </summary>
        [Test]
        public void TestOptionsAccuracy()
        {
            Assert.AreSame(element.Options, element.Options, "The option collection instance should be the same.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>Options</c> property. A <c>BuildElementCollection</c> attribute
        /// with name "options" and nested name "option" should exist. It should be an optional attribute.
        /// </summary>
        [Test]
        public void TestOptionsAttributeAccuracy()
        {
            BuildElementCollectionAttribute attribute = (BuildElementCollectionAttribute)typeof(ServerElement).
                GetProperty("Options").GetCustomAttributes(typeof(BuildElementCollectionAttribute), false)[0];

            // Verify
            Assert.AreEqual("options", attribute.Name, "The name of the attribute must be options.");
            Assert.AreEqual("option", attribute.ChildElementName,
                "The child element name of the attribute must be option.");
            Assert.IsFalse(attribute.Required, "The attribute must be optional.");
        }

        /// <summary>
        /// Tests accuracy of <c>Credentials</c> for the default value. By default, it is not <c>null</c>. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestCredentialsDefaultAccuracy()
        {
            Assert.IsNotNull(element.Credentials, "The default value of credentials should not be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Credentials</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestCredentialsSetAccuracy()
        {
            Credential credential = new Credential();

            // Set the value
            element.Credentials = credential;

            // Verify
            Assert.AreEqual(credential, element.Credentials, "The credentials should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>Credentials</c> property. A <c>BuildElement</c> attribute with
        /// name "credentials" should exist. It should be an optional attribute.
        /// </summary>
        [Test]
        public void TestCredentialsAttributeAccuracy()
        {
            BuildElementAttribute attribute = (BuildElementAttribute)typeof(ServerElement).GetProperty("Credentials").
                GetCustomAttributes(typeof(BuildElementAttribute), false)[0];

            // Verify
            Assert.AreEqual("credentials", attribute.Name, "The name of the attribute must be credentials.");
            Assert.IsFalse(attribute.Required, "The attribute must be optional.");
        }

        /// <summary>
        /// Tests accuracy of <c>SourceFiles</c> for the default value. By default, it is not <c>null</c>. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestSourceFilesDefaultAccuracy()
        {
            Assert.IsNotNull(element.SourceFiles, "The default value of source files should not be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>SourceFiles</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSourceFilesSetAccuracy()
        {
            FileSet files = new FileSet();

            // Set the value
            element.SourceFiles = files;

            // Verify
            Assert.AreEqual(files, element.SourceFiles, "The source files should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of the attributes in the <c>SourceFiles</c> property. A <c>BuildElement</c> attribute with
        /// name "sourcefiles" should exist. It should be an optional attribute.
        /// </summary>
        [Test]
        public void TestSourceFilesAttributeAccuracy()
        {
            BuildElementAttribute attribute = (BuildElementAttribute)typeof(ServerElement).GetProperty("SourceFiles").
                GetCustomAttributes(typeof(BuildElementAttribute), false)[0];

            // Verify
            Assert.AreEqual("sourcefiles", attribute.Name, "The name of the attribute must be sourcefiles.");
            Assert.IsFalse(attribute.Required, "The attribute must be optional.");
        }

        /// <summary>
        /// Tests accuracy of <c>ServerElement()</c>. Since the constructor is trivial, there should be no exception.
        /// </summary>
        [Test]
        public void TestServerElementAccuracy()
        {
            Assert.IsNotNull(new ServerElement(), "A new instance of ServerElement should be created.");
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the server class cannot be found. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceTypeNotFound()
        {
            element.ClassName = "notfound";
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the assembly of the server class cannot be found. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceAssemblyNotFound()
        {
            element.ClassName = "notfound, assembly";
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the constructor of the server class causes error. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceConstructorError()
        {
            element.ClassName = typeof(ErrorServer).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the server class name is invalid. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceClassNameInvalid()
        {
            element.ClassName = "\0";
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the server class is an abstract class. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceClassAbstract()
        {
            element.ClassName = typeof(Server).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the server class is an array class. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceClassArray()
        {
            element.ClassName = typeof(Server[]).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests <c>GetServerInstance()</c> when the class is not a sub-class of <c>Server</c>. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstanceClassNotServer()
        {
            element.ClassName = typeof(object).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests accuracy of <c>GetServerInstance()</c>. An instance of server class should be returned. The instance
        /// should be <c>IISServer</c> instance. The options, start/init server flags, credentials and source files
        /// should be set as in the <c>ServerElement</c> instance.
        /// </summary>
        [Test]
        public void TestGetServerInstanceAccuracy()
        {
            // Create an NAnt project.
            Project project = new Project("../../test_files/default.build", Level.Error);

            // Set the property of the element
            element.StartServer = false;
            element.InitServer = false;
            element.ClassName = typeof(IISServer).FullName;
            element.Project = project;

            // Get the server instance
            Server server = element.GetServerInstance();

            // Verify
            Assert.IsTrue(server is IISServer, "The created server should be IISServer.");
            Assert.IsFalse(server.StartServer, "The start server flag should be false.");
            Assert.IsFalse(server.InitServer, "The init data flag should be false.");
            Assert.AreSame(element.Credentials, server.Credentials,
                "The credentials should be set as the same in the element.");
            Assert.AreSame(element.Options, server.Options, "The options should be set as the same in the element.");
            Assert.AreSame(element.SourceFiles, server.SourceFiles,
                "The source files should be set as the same in the element.");
            Assert.AreSame(element.Project, server.Project,
                "The NAnt project should be set as the same in the element.");
        }

        /// <summary>
        /// Tests accuracy of <c>GetServerInstance()</c>. The created instance should be cached. This is verified by
        /// calling the method twice. The same instance should be returned.
        /// </summary>
        [Test]
        public void TestGetServerInstanceSameAccuracy()
        {
            // Set the property of the element
            element.ClassName = typeof(IISServer).FullName;

            // Verify
            Assert.AreSame(element.GetServerInstance(), element.GetServerInstance(),
                "The created instance should be cached.");
        }
    }
}

