/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NUnit.Framework;
using NAnt.Core;
using NAnt.Core.Types;
using TopCoder.Test.Framework.Servers.Web;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Tests the functionality and error cases of <c>Server</c> class. Since this class is an abstract class, a
    /// concrete implementation <c>IISServer</c> is used in the tests.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ServerTestCase
    {
        /// <summary>
        /// Represents the <c>Server</c> instance used in tests.
        /// </summary>
        private Server server;

        /// <summary>
        /// Sets up the test environment. The test instance is created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            server = new IISServer();
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            server = null;
        }

        /// <summary>
        /// Tests accuracy of <c>Server()</c>. Since the constructor is trivial, there should be no exception.
        /// </summary>
        [Test]
        public void TestServerAccuracy()
        {
            Assert.IsNotNull(new IISServer(), "A new instance of Server should be created.");
        }

        /// <summary>
        /// Tests accuracy of <c>StartServer</c> for the default value. By default, it is <c>true</c>. This is verified
        /// by the property getter.
        /// </summary>
        [Test]
        public void TestStartServerDefaultAccuracy()
        {
            Assert.IsTrue(server.StartServer, "The default value of start server flag should be true.");
        }

        /// <summary>
        /// Tests accuracy of <c>StartServer</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestStartServerSetAccuracy()
        {
            server.StartServer = false;

            // Verify
            Assert.IsFalse(server.StartServer, "The start server flag should be set as false.");
        }

        /// <summary>
        /// Tests accuracy of <c>InitServer</c> for the default value. By default, it is <c>true</c>. This is verified
        /// by the property getter.
        /// </summary>
        [Test]
        public void TestInitServerDefaultAccuracy()
        {
            Assert.IsTrue(server.InitServer, "The default value of init data flag should be true.");
        }

        /// <summary>
        /// Tests accuracy of <c>InitServer</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestInitServerSetAccuracy()
        {
            server.InitServer = false;

            // Verify
            Assert.IsFalse(server.InitServer, "The init data flag should be set as false.");
        }

        /// <summary>
        /// Tests accuracy of <c>Credentials</c> for the default value. By default, it is <c>null</c>. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestCredentialsDefaultAccuracy()
        {
            Assert.IsNull(server.Credentials, "The default value of credentials should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Credentials</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestCredentialsSetAccuracy()
        {
            Credential credential = new Credential();

            // Set the value
            server.Credentials = credential;

            // Verify
            Assert.AreEqual(credential, server.Credentials, "The credentials should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>Project</c> for the default value. By default, it is <c>null</c>. This is verified by
        /// the property getter.
        /// </summary>
        [Test]
        public void TestProjectDefaultAccuracy()
        {
            Assert.IsNull(server.Project, "The default value of project should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Project</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestProjectSetAccuracy()
        {
            Project project = new Project("../../test_files/default.build", Level.Error);

            // Set the value
            server.Project = project;

            // Verify
            Assert.AreEqual(project, server.Project, "The project should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>SourceFiles</c> for the default value. By default, it is <c>null</c>. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestSourceFilesDefaultAccuracy()
        {
            Assert.IsNull(server.SourceFiles, "The default value of source files should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>SourceFiles</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSourceFilesSetAccuracy()
        {
            FileSet files = new FileSet();

            // Set the value
            server.SourceFiles = files;

            // Verify
            Assert.AreEqual(files, server.SourceFiles, "The source files should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>Options</c> for the default value. By default, it is <c>null</c>. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestOptionsDefaultAccuracy()
        {
            Assert.IsNull(server.Options, "The default value of options should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Options</c> for the setting the value. The set value should be retrieved by the
        /// property getter. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestOptionsAccuracy()
        {
            OptionCollection options = new OptionCollection();

            // Set the value
            server.Options = options;

            // Verify
            Assert.AreEqual(options, server.Options, "The options should be set as given.");
        }

        /// <summary>
        /// Tests <c>Timeout</c> property getter when the option collection is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeoutOptionCollectionNotSet()
        {
            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests <c>Timeout</c> property getter when the option "timeout" is missing. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeoutOptionMissing()
        {
            server.Options = new OptionCollection();

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests <c>Timeout</c> property getter when the option "timeout" is not an integer. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeoutOptionNotInteger()
        {
            Option option = new Option();

            option.OptionName = "timeout";
            option.Value = "abc";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests <c>Timeout</c> property getter when the option "timeout" is out of range. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeoutOptionOutOfRange()
        {
            Option option = new Option();

            option.OptionName = "timeout";
            option.Value = "10000000000000000000";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests <c>Timeout</c> property getter when the option "timeout" is a negative integer. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeoutOptionNegative()
        {
            Option option = new Option();

            option.OptionName = "timeout";
            option.Value = "-1";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests accuracy of <c>Timeout</c> property getter. The time out value should be 123, as set in the option
        /// "timeout".
        /// </summary>
        [Test]
        public void TestTimeoutAccuracy()
        {
            Option option = new Option();

            option.OptionName = "timeout";
            option.Value = "123";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual(123, server.Timeout, "The time out value should be defined in option 'timeout'.");
        }

        /// <summary>
        /// Tests <c>Port</c> property getter when the option collection is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPortOptionCollectionNotSet()
        {
            int port = server.Port;
        }

        /// <summary>
        /// Tests <c>Port</c> property getter when the option "port" is missing. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPortOptionMissing()
        {
            server.Options = new OptionCollection();

            int port = server.Port;
        }

        /// <summary>
        /// Tests <c>Port</c> property getter when the option "port" is not an integer. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPortOptionNotInteger()
        {
            Option option = new Option();

            option.OptionName = "port";
            option.Value = "abc";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            int port = server.Port;
        }

        /// <summary>
        /// Tests <c>Port</c> property getter when the option "port" is out of range. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPortOptionOutOfRange()
        {
            Option option = new Option();

            option.OptionName = "port";
            option.Value = "100000";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            int port = server.Port;
        }

        /// <summary>
        /// Tests <c>Port</c> property getter when the option "port" is a negative integer. Expect
        /// ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPortOptionNegative()
        {
            Option option = new Option();

            option.OptionName = "port";
            option.Value = "-1";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            int port = server.Port;
        }

        /// <summary>
        /// Tests accuracy of <c>Port</c> property getter. The port should be 123, as set in the option "port".
        /// </summary>
        [Test]
        public void TestPortAccuracy()
        {
            Option option = new Option();

            option.OptionName = "port";
            option.Value = "123";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual(123, server.Port, "The port should be defined in option 'port'.");
        }

        /// <summary>
        /// Tests <c>Address</c> property getter when the option collection is not set. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestAddressOptionCollectionNotSet()
        {
            string address = server.Address;
        }

        /// <summary>
        /// Tests <c>Address</c> property getter when the option "address" is missing. Expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestAddressOptionMissing()
        {
            server.Options = new OptionCollection();

            string address = server.Address;
        }

        /// <summary>
        /// Tests accuracy of <c>Address</c> property getter. The address should be "localhost", as set in the option
        /// "address".
        /// </summary>
        [Test]
        public void TestAddressAccuracy()
        {
            Option option = new Option();

            option.OptionName = "address";
            option.Value = "localhost";
            server.Options = new OptionCollection();
            server.Options.Add(option);

            // Verify
            Assert.AreEqual("localhost", server.Address, "The address should be defined in option 'address'.");
        }
    }
}

